import sys
import time
import zmq

def main(change_value=None):
    ctx = zmq.Context()
    pub = ctx.socket(zmq.PUSH)
    #pub.setsockopt(zmq.SNDHWM, 1)
    #pub.setsockopt(zmq.LINGER, 1)
    pub.bind("ipc:///tmp/brightml_socket")

    if change_value is not None:
        pub.send_string("add_train " + change_value)
    else:
        while True:
            change = input(">>:)>>: ")
            pub.send_string("add_train " + change)

if __name__ == "__main__":
    main()



import numpy as np
import pandas as pd
from sklearn.pipeline import TransformerMixin
from sklearn.preprocessing import OneHotEncoder
from sklearn.pipeline import make_pipeline
from sklearn.neighbors import KNeighborsRegressor

# from sklearn.ensemble import RandomForestRegressor

from brightml.utils import get_training_data


class DataFrameImputer(TransformerMixin):
    """
    Credits http://stackoverflow.com/a/25562948/1575066
    """

    def __init__(self):
        """Impute missing values.

        Columns of dtype object are imputed with the most frequent value
        in column.

        Columns of other types are imputed with mean of column.

        """
        self.fill = None

    @staticmethod
    def most_frequent(col):
        try:
            return col.value_counts().index[0]
        except IndexError:
            return 0

    @staticmethod
    def get_impute_val(col):
        if col.dtype == np.dtype("O"):
            val = ""
        elif col.dtype == np.dtype(float):
            val = col.mean()
        else:
            val = col.median()
        if isinstance(val, float) and np.isnan(val):
            val = 0
        return val

    def fit(self, X, y=None):
        self.fill = pd.Series([self.get_impute_val(X[c]) for c in X], index=X.columns)
        return self

    def transform(self, X, y=None):
        return X.fillna(self.fill, inplace=False)


class Labelizer(TransformerMixin):
    def __init__(self):
        self.labels = {}

    def get_label(self, x):
        if x not in self.labels:
            # adding +100 makes the columns easier to spot
            self.labels[x] = len(self.labels) + 100
        return self.labels[x]

    def fit(self, X, y=None):
        return self

    def transform(self, X, y=None):
        for x in X:
            X[x] = [self.get_label(y) if X[x].dtype == np.object else y for y in X[x]]
        return X


class SelectLast(TransformerMixin):
    def __init__(self, groupby=("ambient_light", "display_window_name", "datetime_date")):
        self.groupby = groupby
        self.cols = None

    def fit(self, X, y=None):
        self.cols = X.columns
        return self

    def transform(self, X, y=None):
        X = X.groupby(list(self.groupby)).last().reset_index()
        X = X.reindex(self.cols, axis=1)
        X = X.fillna(0)
        return X


def get_pipeline(X):

    return make_pipeline(
        DataFrameImputer(),
        Labelizer(),
        SelectLast(),
        OneHotEncoder(
            n_values="auto",
            handle_unknown="ignore",
            categorical_features=X.dtypes == np.object,
            sparse=False,
        ),
    )


def get_classifier_pipeline(path=None, clf=KNeighborsRegressor(1)):
    data = get_training_data(path)
    if data is None:
        return None, None
    data.drop("datetime_timezone", axis=1, inplace=True)
    pipeline = get_pipeline(data)
    data = pipeline.fit_transform(data)
    X, y = data[:, :-1], data[:, -1]
    print("train.shape", X.shape)
    clf.fit(X, y)
    # print("coef", clf.coef_)
    return pipeline, clf



import os
import simplejson
import pandas as pd

_NONE_PATH = None


def get_brightml_path(path=None):
    global _NONE_PATH
    if path is None:
        if _NONE_PATH is None:
            _USERNAME = os.getenv("SUDO_USER") or os.getenv("USER") or "/."
            path = os.path.expanduser("~" + _USERNAME)
            path = os.path.join(path, ".brightml")
            _NONE_PATH = path
        else:
            path = _NONE_PATH
    return os.path.expanduser(path)


def ensure_path_exists(path):
    if not os.path.exists(path):  # pragma: no cover
        os.makedirs(path)


def get_data_path(path=None):
    data_path = "data.jsonl"
    path = path or get_brightml_path()
    ensure_path_exists(path)
    return os.path.join(path, data_path)


def prep_data(data):
    data = data.sort_values(by="datetime_full")
    if "new_brightness" in data.columns:
        col_sort = [x for x in data.columns if x != "new_brightness"] + ["new_brightness"]
        data = data.reindex(col_sort, axis=1)
    return data


def get_training_data(path=None):
    path = get_data_path(path)
    try:
        data = pd.read_json(path, lines=True)
    except ValueError:
        return None
    return prep_data(data)


def save_sample(data, path=None):
    path = get_data_path(path)
    with open(path, "a") as f:
        f.write(simplejson.dumps(data, ignore_nan=True) + "\n")


def get_brightness_paths():
    base_dir = "/sys/class/backlight"
    return [os.path.join(base_dir, x) + "/" for x in os.listdir(base_dir)]


def ensure_latest_update_path(brightml_path=None):
    brightml_path = brightml_path or get_brightml_path()
    ensure_path_exists(brightml_path)
    last_update_dir = os.path.join(brightml_path, "last_updated/")
    ensure_path_exists(last_update_dir)
    last_update_file = os.path.join(last_update_dir, "update")
    if not os.path.exists(last_update_file):
        with open(last_update_file, "w") as f:
            pass
    return last_update_dir, last_update_file



import sys

# just for demo
def warn(*args, **kwargs):
    pass


import warnings

warnings.warn = warn


def main():
    if "features" in sys.argv[1:]:
        from brightml.brightml import get_features
        import json

        print(json.dumps(get_features(), indent=4))
    else:
        from brightml.brightml import main

        main()




import os
import platform
import numpy as np


def on_power(power_supply_path="/sys/class/power_supply/"):
    """Return `True` when one of the power adapters is online."""
    for supply in os.listdir(power_supply_path):
        # check if supply other than BATTERY is "online"
        if supply.startswith("BAT"):
            continue
        online_path = os.path.join(power_supply_path, supply, "online")
        with open(online_path) as f:
            online = f.read().strip()
            if online == "1":
                return True
    return False


def get_battery_percentage(power_supply_path="/sys/class/power_supply/"):
    """Calculate battery percentage on Linux."""
    available_supply = 0
    max_supply = 0
    for supply in os.listdir(power_supply_path):
        if not supply.startswith("BAT"):
            continue
        # current battery
        charge_now_path = os.path.join(power_supply_path, supply, "charge_now")
        with open(charge_now_path) as f:
            charge_now = int(f.read().strip())
            available_supply += charge_now
        # battery full
        charge_full_path = os.path.join(power_supply_path, supply, "charge_full")
        with open(charge_full_path) as f:
            charge_full = int(f.read().strip())
            max_supply += charge_full
    return available_supply / max_supply * 100


def get_battery_feature():
    """Get battery feature score, currently only on linux."""
    operating_system = platform.system()
    # missing value if not on linux
    if operating_system != "Linux":
        return np.nan
    # if on_power, return highest score (100%)
    if on_power():
        return 100
    # get aggregated battery percentage
    return get_battery_percentage()



import os
import asyncio
import pyinotify

from brightml.brightness import BrightnessManager

from whereami.predict import Predicter

whereami_predicter = Predicter()

bm = BrightnessManager()

wm = pyinotify.WatchManager()  # Watch Manager
mask = pyinotify.ALL_EVENTS  # pyinotify.IN_MODIFY | pyinotify.IN_CLOSE_WRITE  # watched events

D = {}


async def whereami():
    D["whereami"] = whereami_predicter.predict()
    print(D["whereami"])
    await asyncio.sleep(30)
    asyncio.ensure_future(whereami())


async def train():
    await asyncio.sleep(1)
    print("training!")
    # print(D["whereami"])
    # asyncio.ensure_future(whereami())


class EventHandler(pyinotify.ProcessEvent):
    val = None

    def process_IN_CLOSE_WRITE(self, event):
        if self.val is not None:
            # prevent learning
            print("CANCEL")
            self.val.cancel()
        self.val = None
        print("Close write", event.pathname)

    def process_IN_MODIFY(self, event):
        # prevent doule tasks
        if self.val is not None:
            self.val.cancel()
        self.val = asyncio.ensure_future(train())
        print("Modify:", event.pathname)

    # def process_default(self, event):
    #    print(event)


handler = EventHandler()
loop = asyncio.get_event_loop()
loop.create_task(whereami())
notifier = pyinotify.AsyncioNotifier(wm, loop, default_proc_fun=handler)
# notifier = pyinotify.Notifier(wm, handler)

base_dir = "/sys/class/backlight/"
for path in os.listdir(base_dir):
    full_path = os.path.join(base_dir, path)
    print(full_path)
    wm.add_watch(full_path, mask)


async def fuck(x):
    asyncio.sleep(0)
    print("fuck", x)


if __name__ == "__main__":
    from threading import Thread
    from brightml.x11_active_window import main

    t = Thread(target=main, args=[lambda x: loop.create_task(fuck(x))])
    t.start()
    loop.run_forever()
    t.join()



import time
import os

import numpy as np

from brightml.utils import get_brightml_path
from brightml.utils import ensure_latest_update_path


class BrightnessAdapter(object):
    def __init__(self, base_dir, name, brightml_path=None):
        path = base_dir + name + "/"
        self.name = name
        self.path = path
        _, self.update_path = ensure_latest_update_path(brightml_path)
        self.brightness_path = self.path + "brightness"
        self.max_brightness_path = self.path + "max_brightness"
        self.max_brightness = self._read_max_brightness()

    @property
    def brightness(self):
        return self._read_brightness()

    @brightness.setter
    def brightness(self, value):
        self._set_brightness(value)

    def _set_brightness(self, brightness):
        brightness = np.clip(int(brightness), 1, self.max_brightness)
        # print(self.name, self.brightness, brightness)
        with open(self.brightness_path, "w") as f:
            f.write(str(int(brightness)))
        with open(self.update_path, "w") as f:
            pass

    def _read_brightness(self):
        with open(self.brightness_path) as f:
            return int(f.read())

    def _read_max_brightness(self):
        with open(self.max_brightness_path) as f:
            return int(f.read())

    @property
    def is_valid(self):
        # if "intel" not in self.brightness_path:
        #     return False
        v = self.brightness
        self.brightness += 1
        v1 = self.brightness
        self.brightness -= 1
        v2 = self.brightness
        if v != v1 or v != v2:
            return True
        return False

    def __repr__(self):
        return "BrightnessAdapter(path='{}')".format(self.path)


class BrightnessManager(object):
    def __init__(self, base_dir="/sys/class/backlight/", validate=True):
        self.base_dir = base_dir
        self.adapters = [BrightnessAdapter(base_dir, x) for x in os.listdir(base_dir)]
        if validate:
            self.adapters = [x for x in self.adapters if x.is_valid]
        print(self.adapters)

    def get_percentage(self):
        m = 0
        n = 0
        for a in self.adapters:
            m += a.brightness / a.max_brightness
            n += 1
        return m / n

    def adapt(self, percentage_of_max=+5):
        for a in self.adapters:
            v = a.brightness
            m = a.max_brightness
            percentage_of_max = int(percentage_of_max)
            change_value = int(m * (abs(percentage_of_max) / 100))
            if percentage_of_max > 0 and v < m:
                a.brightness += change_value
            elif percentage_of_max < 0 and v > 1:
                a.brightness -= change_value

    def set_by_percentage(self, percentage):
        for a in self.adapters:
            a.brightness = a.max_brightness * (percentage / 100)
            # # external_monitor
            # converted_xrandr = min(1, percentage / 100 * 2)
            # os.system(
            #     "xrandr --output HDMI-0 --brightness {} 2>1 > /dev/null".format(converted_xrandr)
            # )
            # os.system(
            #     "xrandr --output DP-4 --brightness {} 2>1 > /dev/null".format(converted_xrandr)
            # )

    def get_snapshot(self):
        return {x.name: x.brightness for x in self.adapters}

    def set_snapshot(self, snapshot):
        for s, a in zip(snapshot, self.adapters):
            a.brightness = s



import numpy as np

with open("/sys/class/backlight/gmux_backlight/max_brightness") as f:
    MAX_BRIGHTNESS = int(f.read())


def write_brightness(brightness, clip_min=1, clip_max=MAX_BRIGHTNESS):
    brightness = np.clip(int(brightness), clip_min, clip_max)
    with open("/sys/class/backlight/gmux_backlight/brightness", "w") as f:
        f.write(str(brightness))



import time
import asyncio

import pandas as pd


from brightml.features import whereami_predicter, d, get_features
from brightml.utils import save_sample
from brightml.brightness import BrightnessManager
from brightml.pipeline import get_classifier_pipeline
from brightml.file_watcher import watch_fs

from asyncio import ensure_future

from colorama import init, Fore

init()


class Brightml:
    def __init__(self, display):
        self.old_value = None
        self.bm = BrightnessManager()
        self.display = display
        self.pipeline_clf = get_classifier_pipeline(None)

    def adjust_brightness(self, scroll=False):
        if self.pipeline_clf is not None:
            features = get_features()
            new_value = (
                features["ambient_light"],
                features["display_window_name"],
                features["datetime_hour"],
            )
            if self.old_value != new_value or scroll:
                # print("NEW STATE", features)
                if self.pipeline_clf:
                    # t1 = time.time()
                    data = pd.DataFrame([features])
                    import json

                    print(json.dumps(features, sort_keys=True, indent=4))
                    pipeline, clf = self.pipeline_clf
                    if pipeline is None:
                        print(
                            "Change your brightness to create your first data sample (there is no data found)."
                        )
                        return
                    data = pipeline.transform(data)
                    # y is nans in this case
                    X, _ = data[:, :-1], data[:, -1]
                    pred = int(100 * clf.predict(X)[0])
                    tmpl = (
                        "new_brightness="
                        + Fore.GREEN
                        + "{}%"
                        + Fore.RESET
                        + " app="
                        + Fore.GREEN
                        + "{}"
                        + Fore.RESET
                    )
                    print(tmpl.format(pred, features["display_window_class"].split()[-1]))
                    print()
                    self.bm.set_by_percentage(pred)
                else:
                    print("PREDICTING!")
                self.old_value = new_value

    async def retrain(self):
        await asyncio.sleep(0.7)
        features = get_features()
        features["new_brightness"] = self.bm.get_percentage()
        save_sample(features)
        print("RETRAINING")
        self.pipeline_clf = get_classifier_pipeline(None)
        print("RETRAINING DONE")
        self.adjust_brightness()


async def whereami():
    whereami_predicter.predict()
    await asyncio.sleep(30)
    ensure_future(whereami())


from brightml.scrolling import scroll_listen


def main():
    bml = Brightml(d)
    from threading import Thread

    t = Thread(target=d.main_thread_fn, args=[bml.adjust_brightness])
    t.start()
    loop = asyncio.get_event_loop()
    if whereami_predicter is not None:
        loop.create_task(whereami())
    t2 = Thread(target=scroll_listen, args=[bml.adjust_brightness])
    t2.start()
    loop.run_until_complete(watch_fs(bml))
    t.join()
    # t2.join()


if __name__ == "__main__":
    main()



# todo: chargedness of battery

import time
from datetime import datetime
import numpy as np

try:
    from whereami.predict import Predicter

    whereami_predicter = Predicter()
except ImportError:
    print("Could not load `whereami` module")
    whereami_predicter = None

from brightml.battery import get_battery_feature
from brightml.xdisplay import Display

d = Display()


DATE_FORMAT = "%Y-%m-%dT%H:%M:%S%z"


def get_ambient_light():
    try:
        # please, if your ambient light sensor doesn't work, post the path in the issues on github
        with open("/sys/bus/iio/devices/iio:device0/in_illuminance_input") as f:
            ambient = int(f.read())
            return ambient
    except:
        return np.nan


def get_time_features():
    now = datetime.strptime(time.strftime(DATE_FORMAT, time.localtime()), DATE_FORMAT)
    data = {
        "datetime_hour": now.hour,
        "datetime_timezone": str(now.tzinfo),
        "datetime_date": str(now.date()),
        "datetime_full": str(now),
    }
    return data


def get_features():
    if whereami_predicter is None:
        whereami_value = np.nan
    else:
        whereami_value = whereami_predicter.predicted_value or whereami_predicter.predict()
    data = {"ambient_light": get_ambient_light(), "whereami": whereami_value}
    data.update(get_time_features())
    data.update(d.last_value or d.get_window_info())
    data.update({"battery": get_battery_feature()})
    return data



import time
from pynput.mouse import Listener
from cachetools import TTLCache
from brightml.features import d

ttl = TTLCache(5, 0.1)

# TODO
# what needs to be fixed is actually that it also affects the capturing the pixel


def adjust(adjust_brightness_fn):
    def on_scroll(x, y, dx, dy):
        print('Scrolled {0}'.format((x, y)))
        ttl[time.time()] = 1
        if len(ttl) == 4:
            print("scrolled seriously")
            d.get_window_info()
            adjust_brightness_fn(True)

    return on_scroll


def scroll_listen(adjust_brightness_fn):
    while True:
        print("OK")
        with Listener(on_scroll=adjust(adjust_brightness_fn)) as listener:
            listener.join()



import time
import numpy as np
import Xlib
import Xlib.display
import PIL.Image


class Window:
    def __init__(self, window, window_name, window_class):
        self.window = window
        self.window_name = window_name
        self.window_class = window_class

    def get_geometry(self):
        return self.window.get_geometry()

    def get_image(self, *args):
        return self.window.get_image(*args)


class Display:
    def __init__(self):
        self.disp, self.root = self.get_display_and_root()
        self.last_value = None
        self.NET_WM_NAME = self.disp.intern_atom('_NET_WM_NAME')
        self.NET_ACTIVE_WINDOW = self.disp.intern_atom('_NET_ACTIVE_WINDOW')
        self.NET_WM_CLASS = self.disp.intern_atom('_NET_WM_CLASS')
        # Listen for _NET_ACTIVE_WINDOW changes
        self.root.change_attributes(event_mask=Xlib.X.PropertyChangeMask)

    @staticmethod
    def get_display_and_root():
        disp = Xlib.display.Display()
        root = disp.screen().root
        return disp, root

    def get_window(self):
        try:
            window_id = self.root.get_full_property(
                self.NET_ACTIVE_WINDOW, Xlib.X.AnyPropertyType
            ).value[0]
            active_window = self.disp.create_resource_object('window', window_id)
            window_class = " ".join(active_window.get_wm_class())
            window_name = active_window.get_full_property(self.NET_WM_NAME, 0).value
        except:
            return None
        return Window(active_window, window_name, window_class)

    @property
    def default_value(self):
        return {"display_pixel_mean": np.nan, "display_window_name": "", "display_window_class": ""}

    def get_window_info(self):
        active_window = self.get_window()

        if active_window is None:
            return self.default_value

        try:
            geo = active_window.get_geometry()
        except Xlib.error.XError:
            return self.default_value
        # verified okay
        # for gw in [0, 5, 10, 20, 30]:
        #     start_x = max(0, int(gw / 2) - 10)
        #     width = min(20, gw)
        #     print(gw, start_x, width)

        start_x = max(0, int(geo.width / 2) - 50)
        width = min(50, geo.width)

        start_y = max(0, int(geo.height / 2) - 50)
        height = min(50, geo.height)

        # otherwise the screen is not updated yet
        time.sleep(0.07)
        try:
            img = active_window.get_image(
                start_x, start_y, width, height, Xlib.X.ZPixmap, 0xFFFFFFFF
            )
        except Xlib.error.XError:
            return self.last_value

        img = PIL.Image.frombytes("RGB", (width, height), img.data, "raw", "BGRX")
        pixels = np.array(img.getdata(), np.uint8).reshape(img.size[1], img.size[0], 3)

        self.last_value = {
            "display_pixel_mean": pixels.mean(),
            "display_window_name": active_window.window_name.decode("utf8"),
            "display_window_class": active_window.window_class,
        }
        # print(self.last_value)
        return self.last_value

    def main_thread_fn(self, adjust_brightness_fn):
        while True:
            event = self.disp.next_event()
            # Loop through, ignoring events until we're notified of focus/title change
            if event.type != Xlib.X.PropertyNotify:
                continue

            if event.atom in (self.NET_ACTIVE_WINDOW, self.NET_WM_NAME):
                self.get_window_info()
                adjust_brightness_fn()


if __name__ == "__main__":
    from threading import Thread

    d = Display()
    t = Thread(target=d.main_thread_fn, args=[print])
    t.start()
    while True:
        import time

        time.sleep(2)
        print(1)
    t.join()
    print(d.get_window_info())



import time
import zmq

import pandas as pd

from brightml.timer import ZmqTimer, ZmqTimerManager

from brightml.utils import save_sample

from brightml.features import p, d
from brightml.features import get_features
from brightml.brightness import BrightnessManager
from brightml.pipeline import get_classifier_pipeline

bm = BrightnessManager()
pipeline_clf = get_classifier_pipeline(None)
old_value = None

last_add_train = time.time()
last_train_sample = None


def adjust_brightness():
    global last_train_sample
    global old_value
    if last_train_sample is None and pipeline_clf is not None:
        features = get_features()
        new_value = (features["ambient_light"],
                     features["display_window_name"], features["datetime_hour"])
        if old_value != new_value:
            print("NEW STATE", features)
            if pipeline_clf:
                try:
                    #t1 = time.time()
                    data = pd.DataFrame([features])
                    pipeline, clf = pipeline_clf
                    data = pipeline.transform(data)
                    X, _ = data[:, :-1], data[:, -1]
                    pred = int(100 * clf.predict(X)[0])
                    # print("taken", time.time() - t1)
                    print("NEW BRIGHTNESS %", pred)
                    bm.set_by_percentage(pred)
                except ValueError as e:
                    print(e)
                    return
            else:
                print("PREDICTING!")
            old_value = new_value


def add_train(bright_change):
    global last_add_train
    global last_train_sample

    last_add_train = time.time()

    new_brightness = bm.get_percentage()
    features = get_features()
    features["new_brightness"] = new_brightness

    last_train_sample = features
    print("RECEIVED added training", last_train_sample)

    print("OK", bright_change)


def retrain():
    global last_train_sample
    global last_add_train
    global pipeline_clf
    if last_train_sample is not None:
        t1 = time.time()
        if t1 > last_add_train + 0.7:
            save_sample(last_train_sample)
            print("RETRAINING")
            pipeline_clf = get_classifier_pipeline(None)
            print("RETRAINING DONE")
            last_train_sample = None
            last_add_train = None
            last_add_train = t1


COMMANDS = {
    "add_train": add_train,
    "whereami.predict": p.predict,
    "display.get_window_info": d.get_window_info,
    "whereami.refresh": p.refresh
}

# timers in seconds
TIMERS = {
    "adjust_brightness": 0.3,
    "retrain": 0.05,
    "display.get_window_info": 0.3,
    "whereami.predict": 30,
    "whereami.refresh": 300,
}


def main():
    ctx = zmq.Context()
    poller = zmq.Poller()

    timers = ZmqTimerManager()
    timers.add_timer(ZmqTimer(TIMERS["display.get_window_info"], d.get_window_info))
    timers.add_timer(ZmqTimer(TIMERS["adjust_brightness"], adjust_brightness))
    timers.add_timer(ZmqTimer(TIMERS["retrain"], retrain))
    timers.add_timer(ZmqTimer(TIMERS["whereami.predict"], COMMANDS["whereami.predict"]))
    timers.add_timer(ZmqTimer(TIMERS["whereami.refresh"], COMMANDS["whereami.refresh"]))

    grabber = ctx.socket(zmq.PULL)
    grabber.connect("ipc:///tmp/brightml_socket")
    poller.register(grabber, zmq.POLLIN)

    while True:
        timers.check()
        next_interval = timers.get_next_interval()
        socks = dict(poller.poll(next_interval))
        for sock in socks:
            data = sock.recv().decode('utf-8')
            command, *args = data.split()
            if command in COMMANDS:
                fn = COMMANDS[command]
                fn(*args)
            else:
                print("RECEIVED unhandled: %s" % data)


if __name__ == "__main__":
    main()



import time


class ZmqTimerManager(object):
    def __init__(self):
        self.timers = []
        self.next_call = 0

    def add_timer(self, timer):
        self.timers.append(timer)

    def check(self):
        if time.time() > self.next_call:
            for timer in self.timers:
                timer.check()

    def get_next_interval(self):
        if time.time() >= self.next_call:
            call_times = []
            for timer in self.timers:
                call_times.append(timer.get_next_call())
            self.next_call = min(call_times)
            if self.next_call < time.time():
                val = 1
            else:
                val = (self.next_call - time.time()) * 1000
        else:
            val = (self.next_call - time.time()) * 1000
        if val < 1:
            val = 1
        return val


class ZmqTimer(object):
    def __init__(self, interval, callback):
        self.interval = interval
        self.callback = callback
        self.last_call = 0

    def check(self):
        if time.time() > (self.interval + self.last_call):
            self.callback()
            self.last_call = time.time()

    def get_next_call(self):
        return self.last_call + self.interval



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import codecs


from setuptools import setup, find_packages

with open("README.md") as f:
    LONG_DESCRIPTION = f.read()

if sys.argv[-1] == "publish":
    os.system("python setup.py sdist bdist_wheel upload")
    sys.exit()

required = [
    "scipy",
    "numpy",
    "pandas",
    "scikit-learn",
    "python-xlib",
    "Pillow",
    "simplejson",
    "pyinotify",
    "watchdog",
    "cachetools",
    "pynput",
]

extras_require = {'whereami': ['whereami']}

# poor man's version parser
with open("brightml/__init__.py") as f:
    for line in f:
        version = line.strip().split()[-1].strip('"')
        break

setup(
    name='brightml',
    version=version,
    description='Machine Learned Auto brightness',
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    author='Pascal van Kooten',
    author_email='kootenpv@gmail.com',
    url='https://github.com/kootenpv/brightml',
    packages=find_packages(),
    install_requires=required,
    license='MIT',
    entry_points={'console_scripts': ['brightml = brightml.__main__:main']},
    extras_require=extras_require,
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)



import os
import asyncio
import watchdog.observers
from watchdog.observers import Observer

from brightml.utils import get_brightml_path
from brightml.utils import ensure_path_exists
from brightml.utils import get_brightness_paths
from brightml.utils import ensure_latest_update_path

from asyncio import ensure_future

watchdog.observers.inotify_buffer.InotifyBuffer.delay = 0

EVENT_TYPE_MOVED = "moved"
EVENT_TYPE_DELETED = "deleted"
EVENT_TYPE_CREATED = "created"
EVENT_TYPE_MODIFIED = "modified"


class AIOEventHandler(object):
    """An asyncio-compatible event handler."""

    def __init__(self, bml):
        self._loop = asyncio.get_event_loop()
        self.bml = bml
        self.task = None

    @asyncio.coroutine
    def on_modified(self, event):
        # if we wrote the update (meaning we triggered a brightness update)
        # then first wait
        if event.src_path.endswith("update"):
            yield from asyncio.sleep(0.2)
        # if there was an existing task, cancel it
        if self.task is not None:
            self.task.cancel()
        self.task = None
        # if the last event was a modified of one of the brightness files
        # then get features and retrain
        if not event.src_path.endswith("update"):
            self.task = ensure_future(self.bml.retrain())

    def dispatch(self, event):
        _method_map = {EVENT_TYPE_MODIFIED: self.on_modified}
        handlers = [_method_map[event.event_type]]
        for handler in handlers:
            self._loop.call_soon_threadsafe(ensure_future, handler(event))


class AIOWatchdog(object):
    def __init__(self, paths, bml):
        self.observer = Observer()

        if not isinstance(paths, list):
            paths = [paths]

        evh = AIOEventHandler(bml)
        for path in paths:
            self.observer.schedule(evh, path, recursive=False)

    def start(self):
        self.observer.start()

    def stop(self):
        self.observer.stop()
        self.observer.join()


async def watch_fs(bml, brightml_path=None):
    if brightml_path is None:
        brightml_path = get_brightml_path()

    last_update_dir, _ = ensure_latest_update_path(brightml_path)

    monitored_paths = [last_update_dir] + get_brightness_paths()

    print(monitored_paths)
    watch = AIOWatchdog(monitored_paths, bml)
    watch.start()
    await asyncio.sleep(1000)
    watch.stop()
